/* 
** DESCRIPTION: Hin- und Hersenden einer Nachricht
** COMPILE:     mpic++ -O -o pingpong pingpong.cpp
** RUN:         mpiexec -n 2 -machinefile machines $PWD/pingpong <passes> <msg-len>
*/

#include <iostream>
#include <stdlib.h>
#include <mpi.h>

using namespace std;

int main (int argc, char **argv)
{
	int i;
	int passes;
	int size;
	char *buf;
	int myrank, nprocs;
	int namelen;
	char name[MPI_MAX_PROCESSOR_NAME];
	MPI_Status status;
	double start, end;

	/* MPI initialisieren und Argumente setzen */ 
	MPI_Init(&argc, &argv);

	/* Anzahl der Prozesse erfragen */
	MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

	/* Eigenen Rang erfragen */
	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);

	/* Rechnernamen erfragen */
	MPI_Get_processor_name(name, &namelen);

	/* flush, damit Ausgabe sofort kommt */
	cout << "Process " << myrank << "/" << nprocs << "started on " << name << "\n" << flush;

	/* Zahl der Prozesse prfen */
	if (nprocs != 2) {
		if (myrank == 0) {
			cerr << "Need 2 processes, have " << nprocs << "\n";
		}
		MPI_Finalize();
		exit(1);
	}

	/* Zahl der Argumente prfen */
	if (argc < 3) {
		if (myrank == 0) {
			cerr <<	"Usage: mpiexec -n 2 ... pingpong <passes> <msg-len>\n"
				 <<	"       <passes>:  number of messages to be sent\n"
				 <<	"       <msg-len>: message size in Bytes\n";
		}
		MPI_Finalize();
		exit(1);
	}

	/* Achtung: hier fehlen die Prfungen der Argumente! */
	passes = atoi(argv[1]);
	size = atoi(argv[2]);
	buf = new char[size];

	/* Nachricht initialisieren */
	for (i=0; i<size; i++)
		buf[i] = i % 256;

	if (myrank == 0) {
		/* PROZESS 0 */

		/* Aktuelle Zeit bestimmen */
		start = MPI_Wtime();
		
		for (i=0; i<passes; i++) {
			/* Nachricht an Prozess 1 senden, Tag = 42 */
			MPI_Send(buf, size, MPI_CHAR, 1, 42, MPI_COMM_WORLD);

			/* Auf Antwort warten, Tag ist egal */
			MPI_Recv(buf, size, MPI_CHAR, 1, MPI_ANY_TAG, MPI_COMM_WORLD,
					 &status);
		}

		/* Aktuelle Zeit bestimmen */
		end = MPI_Wtime();

		cout << myrank << ": Time for one message: " 
			 << ((end - start) * 1e6 / (2 * passes)) << "us\n";
		cout << myrank << ": Bandwidth: "
			 << (size * 2 * passes / (1024 * 1024 * (end - start))) << "MB/s\n";
	}
	else {
		/* PROZESS 1 */
		
		for (i=0; i<passes; i++) {
			/* Auf Nachricht von Proze 0 warten, Tag ist egal */
			MPI_Recv(buf, size, MPI_CHAR, 0 , MPI_ANY_TAG, MPI_COMM_WORLD,
					 &status);
			
			/* Nachricht an Prozess 0 zurcksenden, Tag = 24 */
			MPI_Send(buf, size, MPI_CHAR, 0, 24, MPI_COMM_WORLD);
		}
	}

	/* MPI geordnet beenden */
	MPI_Finalize();

	return 0;
}

