/* 
** DESCRIPTION: Nachricht im Kreis senden
**              Korrekte Version mit MPI_Irecv
** COMPILE:     mpic++ -O -o ring2 ring2.cpp
** RUN:         mpiexec -n 2 -machinefile machines $PWD/ring2 <msg-len>
**
** Rufen Sie das Programm einmal mit Lnge 100, einmal mit Lnge 1000000 auf!
** Was beobachten Sie?
*/

#include <iostream>
#include <stdlib.h>
#include <mpi.h>

using namespace std;

int main (int argc, char **argv)
{
	int size;
	char *sbuf, *rbuf;
	int myrank, nprocs;
	int namelen;
	char name[MPI_MAX_PROCESSOR_NAME];
	MPI_Status status;
	MPI_Request request;

	/* MPI initialisieren und Argumente setzen */ 
	MPI_Init(&argc, &argv);

	/* Anzahl der Prozesse erfragen */
	MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

	/* Eigenen Rang erfragen */
	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);

	/* Rechnernamen erfragen */
	MPI_Get_processor_name(name, &namelen);

	/* flush, damit Ausgabe sofort kommt */
	cout << "Process " << myrank << "/" << nprocs << "started on " << name << "\n" << flush;

	/* Zahl der Argumente prfen */
	if (argc < 2) {
		if (myrank == 0) {
			cerr <<	"Usage: mpiexec -n <np> ... ring2 <msg-len>\n"
				 <<	"       <msg-len>: message size in Bytes\n";
		}
		MPI_Finalize();
		exit(1);
	}

	/* Achtung: hier fehlt die Prfung des Arguments! */
	size = atoi(argv[1]);
	sbuf = new char[size];
	rbuf = new char[size];


	/* MPI Empfangs-Request aufsetzen */
	cout << "Process " << myrank << ": posting receive ...\n" << flush;
	MPI_Irecv(rbuf, size, MPI_CHAR, (myrank+nprocs-1) % nprocs, 0,
			  MPI_COMM_WORLD, &request);

	/* Nachricht Senden */
	cout << "Process " << myrank << ": sending ...\n" << flush;
	MPI_Send(sbuf, size, MPI_CHAR, (myrank+1) % nprocs, 0,
			 MPI_COMM_WORLD);

	/* Auf Empfang der Nachricht warten */
	cout << "Process " << myrank << ": waiting for receive to complete ...\n" << flush;
	MPI_Wait(&request, &status);

	cout << "Process " << myrank << ": done ...\n" << flush;
	
	/* MPI geordnet beenden */
	MPI_Finalize();

	return 0;
}

