/* 
** DESCRIPTION: Multiplikation Vektor mit Skalar
** COMPILE:     mpic++ -o vecmult vecmult.cpp
** RUN:         mpiexec -n 2 -machinefile machines $PWD/vecmult
**
*/

#include <iostream>
#include <stdlib.h>
#include <mpi.h>

using namespace std;

#define NPROCS  4          // Anzahl der Prozesse
#define N       10000
#define LOCAL_N (N/NPROCS)

int main (int argc, char **argv)
{
	int i;
	int myrank, nprocs;
	int namelen;
	char name[MPI_MAX_PROCESSOR_NAME];

	double a[N], factor, local_a[LOCAL_N];


	/* MPI initialisieren und Argumente setzen */ 
	MPI_Init(&argc, &argv);

	/* Anzahl der Prozesse erfragen */
	MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

	/* Eigenen Rang erfragen */
	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);

	/* Rechnernamen erfragen */
	MPI_Get_processor_name(name, &namelen);

	/* flush, damit Ausgabe sofort kommt */
	cout << "Process " << myrank << "/" << nprocs << "started on " << name << "\n" << flush;

	if (LOCAL_N * nprocs != N) {
		if (myrank == 0) {
			cerr << "Illegal number of processes (need " << NPROCS << ")\n";
		}
		MPI_Finalize();
		exit(1);
	}


	if (myrank == 0) {
		/* Initialisiere a und factor. (Typisch: lesen aus Datei) */
		for (i=0; i<N; i++) {
			a[i] = i/10.0;
		}
		factor = 10.0;
	}

	/* Proze 0 sendet factor an alle Prozesse */
	MPI_Bcast(&factor, 1, MPI_DOUBLE, 0, MPI_COMM_WORLD);

	/* Proze 0 verteilt a auf alle Prozesse */
	MPI_Scatter(a, LOCAL_N, MPI_DOUBLE, local_a, LOCAL_N,
				MPI_DOUBLE, 0, MPI_COMM_WORLD);

	/* Lokalen Teil von a mit factor multiplizieren */
	for (i=0; i<LOCAL_N; i++)
		local_a[i] *= factor;

	/* Ergebnis wieder in Proze 0 aufsammeln */
	MPI_Gather(local_a, LOCAL_N, MPI_DOUBLE, a, LOCAL_N, 
			   MPI_DOUBLE, 0, MPI_COMM_WORLD);

	/*
	** Hinweis: dieses Programm ist natrlich nicht effizient, da zuwenig
	** berechnet wird. Stellen Sie sich aber vor, da statt der Multiplikation
	** eine komplexe Funktion berechnet werden mu, die von a[i] und factor
	** abhngt.
	*/

	if (myrank == 0) {
		/* Ausgabe von a. (Typisch: schreiben in Datei) */
		for (i=0; i<N; i+=N/20) {
			cout << "a[" << i << "] = " << a[i] << "\n";
		}
	}

 
	/* MPI geordnet beenden */
	MPI_Finalize();

	return 0;
}

